@@ -1,7 +1,7 @@
 #include "mltaln.h"
 
 //static FILE *fftfp;
-static int n20or4or2;
+static TLS int n20or4or2;
 
 #define KEIKA 0
 #define RND   0
@@ -42,7 +42,7 @@ static void vec_init2( Fukusosuu **result, char *seq, double eff, int st, int ed
 
 static void seq_vec_2( Fukusosuu *result, double *score, double incr, char *seq )
 {
-	static int n;
+	static TLS int n;
 	for( ; *seq; result++ )
 	{
 		n = amino_n[(int)*seq++];
@@ -123,9 +123,9 @@ static int segcmp( void *ptr1, void *ptr2 )
 static void mymergesort( int first, int last, Segment **seg )
 {
 	int middle;
-	static int i, j, k, p;
-	static int allo = 0;
-	static Segment **work = NULL;
+	static TLS int i, j, k, p;
+	static TLS int allo = 0;
+	static TLS Segment **work = NULL;
 	if( last > allo )
 	{
 		allo = last;
@@ -164,36 +164,36 @@ float Falign_localhom( char  **seq1, char  **seq2,
    // prevalloclen ha iranai.
 	int i, j, k, l, m, maxk;
 	int nlen, nlen2, nlen4;
-	static int crossscoresize = 0;
-	static char **tmpseq1 = NULL;
-	static char **tmpseq2 = NULL;
-	static char **tmpptr1 = NULL;
-	static char **tmpptr2 = NULL;
-	static char **tmpres1 = NULL;
-	static char **tmpres2 = NULL;
-	static char **result1 = NULL;
-	static char **result2 = NULL;
+	static TLS int crossscoresize = 0;
+	static TLS char **tmpseq1 = NULL;
+	static TLS char **tmpseq2 = NULL;
+	static TLS char **tmpptr1 = NULL;
+	static TLS char **tmpptr2 = NULL;
+	static TLS char **tmpres1 = NULL;
+	static TLS char **tmpres2 = NULL;
+	static TLS char **result1 = NULL;
+	static TLS char **result2 = NULL;
 #if RND
-	static char **rndseq1 = NULL;
-	static char **rndseq2 = NULL;
+	static TLS char **rndseq1 = NULL;
+	static TLS char **rndseq2 = NULL;
 #endif
-	static Fukusosuu **seqVector1 = NULL;
-	static Fukusosuu **seqVector2 = NULL;
-	static Fukusosuu **naiseki = NULL;   
-	static Fukusosuu *naisekiNoWa = NULL; 
-	static double *soukan = NULL;
-	static double **crossscore = NULL;
+	static TLS Fukusosuu **seqVector1 = NULL;
+	static TLS Fukusosuu **seqVector2 = NULL;
+	static TLS Fukusosuu **naiseki = NULL;   
+	static TLS Fukusosuu *naisekiNoWa = NULL; 
+	static TLS double *soukan = NULL;
+	static TLS double **crossscore = NULL;
 	int nlentmp;
-	static int *kouho = NULL;
-	static Segment *segment = NULL;
-	static Segment *segment1 = NULL;
-	static Segment *segment2 = NULL;
-	static Segment **sortedseg1 = NULL;
-	static Segment **sortedseg2 = NULL;
-	static int *cut1 = NULL;
-	static int *cut2 = NULL;
-	static char *sgap1, *egap1, *sgap2, *egap2;
-	static int localalloclen = 0;
+	static TLS int *kouho = NULL;
+	static TLS Segment *segment = NULL;
+	static TLS Segment *segment1 = NULL;
+	static TLS Segment *segment2 = NULL;
+	static TLS Segment **sortedseg1 = NULL;
+	static TLS Segment **sortedseg2 = NULL;
+	static TLS int *cut1 = NULL;
+	static TLS int *cut2 = NULL;
+	static TLS char *sgap1, *egap1, *sgap2, *egap2;
+	static TLS int localalloclen = 0;
 	int lag;
 	int tmpint;
 	int count, count0;
