@@ -39,7 +39,9 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
                         int **ijp )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, limk;
-	char gap[] = "-";
+//	char gap[] = "-";
+	char *gap;
+	gap = newgapstr;
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
@@ -126,19 +128,19 @@ float G__align11( char **seq1, char **seq2, int alloclen )
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static float mi, *m;
-	static int **ijp;
-	static int mpi, *mp;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static char **mseq1;
-	static char **mseq2;
-	static char **mseq;
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS int **ijp;
+	static TLS int mpi, *mp;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS char **mseq1;
+	static TLS char **mseq2;
+	static TLS char **mseq;
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 
 	wm = 0.0;
 
@@ -433,14 +435,14 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 	float *mjpt, *prept, *curpt;
 //	int *mpjpt;
 #endif
-	static float mi, *m;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 
 	wm = 0.0;
 
