@@ -10,7 +10,7 @@
 #define DPTANNI 100
 
 
-static int reccycle = 0;
+static TLS int reccycle = 0;
 
 
 static void match_calc( float *match, float **cpmx1, float **cpmx2, int i1, int lgth2, float **floatwork, int **intwork, int initialize )
@@ -468,8 +468,8 @@ static float MSalignmm_rec( int icyc, int jcyc, double *eff1, double *eff2, char
 	float firstm;
 	int firstmp;
 #if STOREWM
-	static char ttt1[50000];
-	static char ttt2[50000];
+	static TLS char ttt1[50000];
+	static TLS char ttt2[50000];
 #endif
 
 	ogcp1 = gapinfo[0] + ist;
