@@ -9,8 +9,8 @@
 #define FASTMATCHCALC 1
 
 
-static float **impmtx = NULL;
-static int impalloclen = 0;
+static TLS float **impmtx = NULL;
+static TLS int impalloclen = 0;
 float imp_match_out_sc( int i1, int j1 )
 {
 //	fprintf( stderr, "imp+match = %f\n", impmtx[i1][j1] * fastathreshold );
@@ -79,8 +79,8 @@ void imp_match_init_strict( float *imp, int clus1, int clus2, int lgth1, int lgt
 	float effij_kozo;
 	double effijx;
 	char *pt, *pt1, *pt2;
-	static char *nocount1 = NULL;
-	static char *nocount2 = NULL;
+	static TLS char *nocount1 = NULL;
+	static TLS char *nocount2 = NULL;
 	LocalHom *tmpptr;
 
 	if( impalloclen < lgth1 + 2 || impalloclen < lgth2 + 2 )
@@ -295,11 +295,11 @@ for( i = 0; i<impalloclen; i++ )
 void imp_match_init( float *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, LocalHom ***localhom )
 {
 	int dif, i, j, k1, k2, tmpint, start1, start2, end1, end2;
-	static int impalloclen = 0;
+	static TLS int impalloclen = 0;
 	char *pt;
 	int allgap;
-	static char *nocount1 = NULL;
-	static char *nocount2 = NULL;
+	static TLS char *nocount1 = NULL;
+	static TLS char *nocount2 = NULL;
 
 	if( impalloclen < lgth1 + 2 || impalloclen < lgth2 + 2 )
 	{
@@ -885,25 +885,25 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static float mi, *m;
-	static int **ijp;
-	static int mpi, *mp;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static char **mseq1;
-	static char **mseq2;
-	static char **mseq;
-	static float *ogcp1;
-	static float *ogcp2;
-	static float *fgcp1;
-	static float *fgcp2;
-	static float **cpmx1;
-	static float **cpmx2;
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS int **ijp;
+	static TLS int mpi, *mp;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS char **mseq1;
+	static TLS char **mseq2;
+	static TLS char **mseq;
+	static TLS float *ogcp1;
+	static TLS float *ogcp2;
+	static TLS float *fgcp1;
+	static TLS float *fgcp2;
+	static TLS float **cpmx1;
+	static TLS float **cpmx2;
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 	float fpenalty = (float)penalty;
 	float *fgcp2pt;
 	float *ogcp2pt;
@@ -1338,25 +1338,25 @@ float A__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, int
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static float mi, *m;
-	static int **ijp;
-	static int mpi, *mp;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static char **mseq1;
-	static char **mseq2;
-	static char **mseq;
-	static float *ogcp1;
-	static float *ogcp2;
-	static float *fgcp1;
-	static float *fgcp2;
-	static float **cpmx1;
-	static float **cpmx2;
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS int **ijp;
+	static TLS int mpi, *mp;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS char **mseq1;
+	static TLS char **mseq2;
+	static TLS char **mseq;
+	static TLS float *ogcp1;
+	static TLS float *ogcp2;
+	static TLS float *fgcp1;
+	static TLS float *fgcp2;
+	static TLS float **cpmx1;
+	static TLS float **cpmx2;
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 	float *fgcp2pt;
 	float *ogcp2pt;
 	float fgcp1va;
@@ -1753,40 +1753,3 @@ fprintf( stderr, "\n" );
 	return( wm );
 }
 
-float translate_and_Calign( char **mseq1, char **mseq2, double *effarr1, double *effarr2, int clus1, int clus2, int alloclen )
-{
-    int i;
-    float wm;
-    char **result;
-    char *seq = NULL, **aseq = NULL;
-    double *effarr = NULL;
-    int nseq = 0;
-	int resultlen;
-
-    if     ( clus1 == 1 && clus2 != 1 ) 
-    {
-        seq = mseq1[0]; aseq = mseq2; effarr = effarr2; nseq = clus2+1;
-#if 0
-		printf( "effarr in transl... = \n" );
-		for( i=0; i<clus2; i++ ) printf( "%f ", effarr2[i] );
-#endif
-    }
-    else if( clus1 != 1 && clus2 == 1 ) 
-    {
-        seq = mseq2[0]; aseq = mseq1; effarr = effarr1; nseq = clus1+1;
-    }
-    else ErrorExit( "ERROR in translate_and_Calign" );
-
-    result = Calignm1( &wm, aseq, seq, effarr, nseq-2, 0 );
-
-	resultlen = strlen( result[0] );
-	if( alloclen < resultlen || resultlen > N )
-	{
-		fprintf( stderr, "alloclen=%d, resultlen=%d, N=%d\n", alloclen, resultlen, N );
-		ErrorExit( "LENGTH OVER!\n" );
-	}
-    for( i=0; i<nseq-1; i++ ) strcpy( aseq[i], result[i] );
-    strcpy( seq, result[nseq-1] );
-
-    return( 0.0 );
-}
