@@ -74,7 +74,7 @@ extern int fastconjuction_noname( int *memlist, char **seq, char **aseq, double
 extern int fastconjuction_noweight( int *memlist, char **seq, char **aseq, double *peff, char *d );
 extern int conjuctionfortbfast( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char *d );
 extern int conjuctionfortbfast_kozo( double *tmptmp, char **pair, int s, char **seq, char **aseq, double *peff, double *eff, double *peff_kozo, double *eff_kozo, char *d );
-extern int conjuction( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char name[M][B], char aname[M][B], char *d );
+extern int conjuction( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char **name, char **aname, char *d );
 extern void floatdelete( float **cpmx, int d, int len );
 extern void chardelete( char *seq, int d );
 extern int RootBranchNode( int nseq, int ***topol, int step, int branch );
@@ -88,7 +88,7 @@ extern int shrinklocalhom( char **pair, int s1, int s2, LocalHom **localhom, Loc
 extern int msshrinklocalhom( char **pair, int s1, int s2, LocalHom **localhom, LocalHom ***localhomshrink );
 extern int fastshrinklocalhom( int *mem1, int *mem2, LocalHom **localhom, LocalHom ***localhomshrink );
 extern int msfastshrinklocalhom( int *mem1, int *mem2, LocalHom **localhom, LocalHom ***localhomshrink );
-extern int TreeDependentIteration( int locnjob, char name[M][B], int nlen[M], char **aseq, char **bseq, int ***topol, double **len, int alloclen, LocalHom **localhomtable, RNApair ***single, int nkozo, char *kozoarivec );
+extern int TreeDependentIteration( int locnjob, char **name, int nlen[M], char **aseq, char **bseq, int ***topol, double **len, int alloclen, LocalHom **localhomtable, RNApair ***single, int nkozo, char *kozoarivec );
 extern void checkMinusLength( int nseq, double **len );
 extern void negativeMember2( int *mem, int *query, int locnseq );
 extern int *negativeMember( int *query, int locnseq );
@@ -190,6 +190,7 @@ extern void kake2hiku( char *str );
 extern void readDataforgaln( FILE *fp, char **name, int *nlen, char **seq );
 extern void readData( FILE *fp, char name[][B], int nlen[], char **seq );
 extern void readData_pointer( FILE *fp, char **name, int *nlen, char **seq );
+extern void readData_pointer2( FILE *fp, int nseq, char **name, int *nlen, char **seq );
 extern void readData_varlen( FILE *fp, char **name, int *nlen, char **seq );
 extern int countATGC( char *s, int *total );
 extern void getnumlen( FILE *fp );
@@ -202,11 +203,13 @@ extern void readhat2_floathalf( FILE *fp, int nseq, char name[M][B], float **mtx
 extern void readhat2_floathalf_pointer( FILE *fp, int nseq, char **name, float **mtx );
 extern void readhat2_float( FILE *fp, int nseq, char name[M][B], float **mtx );
 extern void readhat2_int( FILE *fp, int nseq, char name[M][B], int **mtx );
+extern void readhat2_pointer( FILE *fp, int nseq, char **name, double **mtx );
 extern void readhat2( FILE *fp, int nseq, char name[M][B], double **mtx );
 extern void WriteFloatHat2_pointer_halfmtx( FILE *hat2p, int locnjob, char **name, float **mtx );
 extern void WriteFloatHat2( FILE *hat2p, int locnjob, char name[M][B], float **mtx );
 extern void WriteHat2_int( FILE *hat2p, int locnjob, char name[M][B], int **mtx );
 extern void WriteHat2( FILE *hat2p, int locnjob, char name[M][B], double **mtx );
+extern void WriteHat2_pointer( FILE *hat2p, int locnjob, char **name, double **mtx );
 extern int ReadFasta_sub( FILE *fp, double *dis, int nseq, char name[M][B] );
 extern int ReadSsearch( FILE *fp, double *dis, int nseq, char name[M][B] );
 extern int ReadBlastm7( FILE *fp, double *dis, int qmem, char name[M][B], LocalHom *localhomlist );
@@ -222,7 +225,7 @@ extern int ReadFasta3( FILE *fp, double *dis, int nseq, char name[M][B] );
 extern int ReadFasta( FILE *fp, double *dis, int nseq, char name[M][B] );
 extern int ReadOpt( FILE *fp, int opt[M], int nseq, char name[M][B] );
 extern int ReadOpt2( FILE *fp, int opt[M], int nseq, char name[M][B] );
-extern int writePre( int nseq, char name[][B], int nlen[M], char **aseq, int force );
+extern int writePre( int nseq, char **name, int nlen[M], char **aseq, int force );
 extern void readOtherOptions( int *ppidptr, int *fftThresholdptr, int *fftWinSizeptr );
 extern void initSignalSM( void );
 extern void initFiles( void );
@@ -281,13 +284,14 @@ extern void foldalignedrna( int clus1, int clus2, char **mseq1, char **mseq2, do
 void readmccaskill( FILE *fp, RNApair **pairprob, int length );
 void makegrouprna( RNApair ***group, RNApair ***all, int *memlist );
 void makegrouprnait( RNApair ***group, RNApair ***all, char **pair, int s );
-extern void fixed_musclesupg_float_realloc_nobk_halfmtx( int nseq, float **eff, int ***topol, float **len );
-extern void loadtree( int nseq, int ***topol, float **len, char **name, int *nlen );
+extern void fixed_musclesupg_float_realloc_nobk_halfmtx( int nseq, float **eff, int ***topol, float **len, Treedep * );
+extern void loadtree( int nseq, int ***topol, float **len, char **name, int *nlen, Treedep * );
 extern void loadtop( int nseq, float **eff, int ***topol, float **len );
-extern void fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen );
+extern void fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep * );
+extern void fixed_musclesupg_double_treeout( int nseq, double **eff, int ***topol, double **len, char **name );
 extern void imp_match_init_strict( float *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1kozo, double*eff2kozo, LocalHom ***localhom, int forscore );
 extern void miyataout_reorder_pointer( FILE *fp, int locnjob, int nlenmax, char **name, int *nlen, char **aseq, int *order );
-extern void veryfastsupg_double_outtree( int nseq, double **eff, int ***topol, double **len, char name[][B] );
+extern void veryfastsupg_double_outtree( int nseq, double **eff, int ***topol, double **len, char **name );
 extern void cpmx_ribosum( char **seq, char **seqr, char *dir, float **cpmx, double *eff, int lgth, int clus );
 extern void rnaalifoldcall( char **seq, int nseq, RNApair **pairprob );
 extern void readpairfoldalign( FILE *fp, char *seq1, char *seq2, char *aln1, char *aln2, int q1, int q2, int *of1, int *of2, int sumlen );
@@ -296,3 +300,14 @@ extern void assignstrweight( int nseq, double *strweight, Node *stopol, int ***t
 extern void cutData( FILE *, int **, char **, int * );
 extern void cutAlignment( FILE *, int **, char **, int *, char **, char ** );
 extern void catData( FILE * );
+extern void getnumlen_nogap_outallreg_web( FILE *fp, FILE *ofp, int *nlenminpt, int *isalignedpt );
+extern void getnumlen_nogap_outallreg( FILE *fp, int *nlenminpt );
+extern double plainscore( int nseq, char **seq );
+extern void eq2dash( char *s );
+extern void findnewgaps( int n, char **seq, int *gaplen );
+extern void findcommongaps( int, char **, int * );
+extern void adjustgapmap( int, int *, char * );
+extern void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex2, int *gaplen, int alloclen );
+extern void restorecommongaps( int n, char **seq, int *top0, int *top1, int *gaplen, int alloclen );
+extern int samemember( int *mem, int *cand );
+extern int includemember( int *mem, int *cand );
