@@ -16,9 +16,12 @@
 #include <ctype.h>
 #include "mtxutl.h"
 #include <float.h>
+#ifdef enablemultithread
+#include <pthread.h>
+#endif
 
-#define VERSION "6.717b"
-#define SHOWVERSION fprintf( stderr, "%s (%s) Version " VERSION " alg=%c, model=%s\n", progName( argv[0] ), (dorp=='d')?"nuc":"aa", alg, modelname )
+#define VERSION "6.815b"
+#define SHOWVERSION fprintf( stderr, "%s (%s) Version " VERSION " alg=%c, model=%s\n%d thread(s)\n", progName( argv[0] ), (dorp=='d')?"nuc":"aa", alg, modelname, nthread )
 
 #define FFT_THRESHOLD  80
 #define FFT_WINSIZE_P   20
@@ -124,6 +127,8 @@ extern char *swopt;
 extern int fftkeika;
 extern int score_check;
 extern char *inputfile;
+extern char *addfile;
+extern int addprofile;
 extern float consweight_multi;
 extern float consweight_rna;
 extern char RNAscoremtx;
@@ -261,8 +266,37 @@ typedef struct _RNApair
 	float bestscore;
 } RNApair;
 
+typedef struct _Treedep
+{
+	int child0;
+	int child1;
+	int done;
+} Treedep;
+
 
 #include "fft.h"
 #include "dp.h"
 #include "functions.h"
 
+#ifdef enablemultithread
+#define TLS __thread
+#else
+#define TLS 
+#endif
+
+extern TLS int commonAlloc1;
+extern TLS int commonAlloc2;
+extern TLS int **commonIP;
+extern TLS int **commonJP;
+extern int nthread;
+extern int randomseed;
+extern int parallelizationstrategy;
+#define BESTFIRST 0
+#define BAATARI0 1
+#define BAATARI1 2
+#define BAATARI2 3
+
+extern int scoreout;
+extern int outnumber;
+
+extern char *newgapstr;
