@@ -244,16 +244,20 @@ static int checkcbeta( FILE *fp )
 }
 
 
-static float callrash( char **mseq1, char **mseq2, double *equiv, char *fname1, char *chain1, char *fname2, char *chain2, int alloclen )
+static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *equiv, char *fname1, char *chain1, char *fname2, char *chain2, int alloclen )
 {
 	FILE *fp;
 	int res;
-	static char com[10000];
+	char com[10000];
+	char dirname[1000];
 	float value;
 	char cachedir[10000];
 	char cachefile[10000];
 	int runnow;
 
+	sprintf( dirname, "_rashtmp_%d-%d", mem1, mem2 );
+	sprintf( com, "rm -rf %s; mkdir %s", dirname, dirname );
+	system( com );
 
 	if( usecache )
 	{
@@ -287,9 +291,9 @@ static float callrash( char **mseq1, char **mseq2, double *equiv, char *fname1,
 		res = system( com );
 #endif
 #if 1  // 091127, pdp nai!
-		sprintf( com, "env PATH=%s PDP_ASH.pl --qf %s.pdb --qc %s --tf %s.pdb --tc %s > _rashout 2>_dum", whereispairalign, fname1, chain1, fname2, chain2 );
+		sprintf( com, "cd %s; %s/PDP_ASH.pl --qf ../%s.pdb --qc %s --tf ../%s.pdb --tc %s > _rashout 2>_dum", dirname, whereispairalign, fname1, chain1, fname2, chain2 );
 #else
-		sprintf( com, "env PATH=%s rash --qf %s.pdb --qc %s --tf %s.pdb --tc %s > _rashout 2>_dum", whereispairalign, fname1, chain1, fname2, chain2 );
+		sprintf( com, "cd %s; %s/rash --qf ../%s.pdb --qc %s --tf ../%s.pdb --tc %s > _rashout 2>_dum", dirname, whereispairalign, fname1, chain1, fname2, chain2 );
 #endif
 		fprintf( stderr, "command = %s\n", com );
 		res = system( com );
@@ -303,7 +307,7 @@ static float callrash( char **mseq1, char **mseq2, double *equiv, char *fname1,
 	else
 	{
 		fprintf( stderr, "Use cache!\n" );
-		sprintf( com, "grep -v successful %s > _rashout", cachefile );
+		sprintf( com, "grep -v successful %s > %s/_rashout", dirname, cachefile );
 		system( com );
 	}
 
@@ -315,7 +319,8 @@ static float callrash( char **mseq1, char **mseq2, double *equiv, char *fname1,
 		system( com );
 	}
 
-	fp = fopen( "_rashout", "r" );
+	sprintf( com, "%s/_rashout", dirname );
+	fp = fopen( com, "r" );
 	if( !fp )
 	{
 		fprintf( stderr, "Cannot open _rashout\n" );
@@ -324,8 +329,11 @@ static float callrash( char **mseq1, char **mseq2, double *equiv, char *fname1,
 
 	readrash( fp, *mseq1, *mseq2, equiv );
 
+
 	fclose( fp );
 
+	sprintf( com, "rm -rf %s", dirname );
+
 //	fprintf( stderr, "*mseq1 = %s\n", *mseq1 );
 //	fprintf( stderr, "*mseq2 = %s\n", *mseq2 );
 
@@ -422,24 +430,29 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 	alreadyoutput = AllocateIntVec( njob );
 	for( i=0; i<njob; i++ ) alreadyoutput[i] = 0;
 
-	for( i=0; i<istr; i++ )
+#if 1
 	{
-		fprintf( stderr, "i=%d\n", i );
-		(*seqpt)[i][0] = 0;
-
-		(*mseq1pt)[0] = (*seqpt)[i];
-		(*mseq2pt)[0] = dumseq;
-
-		callrash( *mseq1pt, *mseq2pt, *equivpt, (*strfiles)[i], (*chainids)[i], (*strfiles)[i], (*chainids)[i], *alloclenpt );
-		fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, (*strfiles)[i], (*chainids)[i], (*seqpt)[i] );
-		alreadyoutput[i] = 1;
+		for( i=0; i<istr; i++ )
+		{
+			fprintf( stderr, "i=%d\n", i );
+			(*seqpt)[i][0] = 0;
+	
+			(*mseq1pt)[0] = (*seqpt)[i];
+			(*mseq2pt)[0] = dumseq;
+	
+			callrash( i, i, *mseq1pt, *mseq2pt, *equivpt, (*strfiles)[i], (*chainids)[i], (*strfiles)[i], (*chainids)[i], *alloclenpt );
+			fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, (*strfiles)[i], (*chainids)[i], (*seqpt)[i] );
+			alreadyoutput[i] = 1;
+		}
 	}
+#endif
 }
 
 void arguments( int argc, char *argv[] )
 {
     int c;
 
+	nthread = 0;
 	usecache = 0;
 	scale = 1.0;
 	equivthreshold = 5;
@@ -568,6 +581,10 @@ void arguments( int argc, char *argv[] )
 					scale = atof( *++argv );
 					--argc;
 					goto nextoption;
+				case 'C':
+					nthread = atoi( *++argv );
+					--argc;
+					goto nextoption;
 				case 'c':
 					usecache = 1;
 					break;
@@ -588,85 +605,10 @@ void arguments( int argc, char *argv[] )
 				case 'e':
 					fftscore = 0;
 					break;
-#if 0
-				case 'O':
-					fftNoAnchStop = 1;
-					break;
-#endif
-				case 'Q':
-					calledByXced = 1;
-					break;
-				case 'x':
-					disp = 1;
-					break;
-#if 0
-				case 'a':
-					alg = 'a';
-					break;
-#endif
-				case 'S':
-					alg = 'S';
-					break;
-				case 'L':
-					alg = 'L';
-					break;
-				case 'B':
-					alg = 'B';
-					break;
-				case 'T':
-					alg = 'T';
-					break;
-				case 'H':
-					alg = 'H';
-					break;
-				case 'M':
-					alg = 'M';
 					break;
 				case 'R':
 					alg = 'R';
 					break;
-				case 'N':
-					alg = 'N';
-					break;
-				case 'K':
-					alg = 'K';
-					break;
-				case 'A':
-					alg = 'A';
-					break;
-				case 'V':
-					alg = 'V';
-					break;
-				case 'C':
-					alg = 'C';
-					break;
-				case 'F':
-					use_fft = 1;
-					break;
-				case 'v':
-					tbrweight = 3;
-					break;
-				case 'y':
-					divpairscore = 1;
-					break;
-/* Modified 01/08/27, default: user tree */
-				case 'J':
-					tbutree = 0;
-					break;
-/* modification end. */
-#if 0
-				case 'z':
-					fftThreshold = atoi( *++argv );
-					--argc; 
-					goto nextoption;
-				case 'w':
-					fftWinSize = atoi( *++argv );
-					--argc;
-					goto nextoption;
-				case 'Z':
-					checkC = 1;
-					break;
-#endif
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
@@ -721,7 +663,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 	char *hat2file = "hat2";
 	LocalHom **localhomtable, *tmpptr;
 	static char **pair;
-	int intdum;
+//	int intdum;
 	double bunbo;
 	char **checkseq;
 
@@ -784,87 +726,17 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 	{
 		fprintf( stderr, "% 5d / %d\r", i, njob );
 
-
 		for( j=i+1; j<njob; j++ )
 		{
-
-
-#if 0
-			if( strlen( seq[i] ) == 0 || strlen( seq[j] ) == 0 )
-			{
-				distancemtx[i][j] = pscore;
-				continue;
-			}
-#endif
-
 			strcpy( aseq[i], seq[i] );
 			strcpy( aseq[j], seq[j] );
 			clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
 			clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
-	//		fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
-	//		fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
 	
-#if 0
-			fprintf( stderr, "group1 = %.66s", indication1 );
-			fprintf( stderr, "\n" );
-			fprintf( stderr, "group2 = %.66s", indication2 );
-			fprintf( stderr, "\n" );
-#endif
-//			for( l=0; l<clus1; l++ ) fprintf( stderr, "## STEP-eff for mseq1-%d %f\n", l, effarr1[l] );
-	
-#if 1
-			if( use_fft )
-			{
-				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
-				off1 = off2 = 0;
-			}
-			else
-#endif
-			{
-				switch( alg )
-				{
-					case( 'a' ):
-						pscore = Aalign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen );
-						off1 = off2 = 0;
-						break;
-					case( 'A' ):
-						pscore = G__align11( mseq1, mseq2, alloclen );
-						off1 = off2 = 0;
-						break;
-					case( 'R' ):
-						fprintf( stderr, "  Calling PDP_ASH.pl %d-%d/%d    \r", i+1, j+1, njob );
-						pscore = callrash( mseq1, mseq2, equiv, strfiles[i], chainids[i], strfiles[j], chainids[j], alloclen );
-						off1 = off2 = 0;
-						break;
-#if 0
-					case( 'V' ):
-						pscore = VAalign11( mseq1, mseq2, alloclen, &off1, &off2, localhomtable[i]+j );
-						fprintf( stderr, "i,j = %d,%d, score = %f\n", i,j, pscore );
-						break;
-					case( 'S' ):
-						fprintf( stderr, "aligning %d-%d\n", i, j );
-						pscore = suboptalign11( mseq1, mseq2, alloclen, &off1, &off2, localhomtable[i]+j );
-						fprintf( stderr, "i,j = %d,%d, score = %f\n", i,j, pscore );
-						break;
-#endif
-					case( 'N' ):
-						pscore = genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
-//						fprintf( stderr, "pscore = %f\n", pscore );
-						break;
-					case( 'K' ):
-						pscore = genG__align11( mseq1, mseq2, alloclen );
-//						fprintf( stderr, "pscore = %f\n", pscore );
-						off1 = off2 = 0;
-						break;
-					case( 'L' ):
-						pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
-//						fprintf( stderr, "pscore (1) = %f\n", pscore );
-//						pscore = (float)naivepairscore11( *mseq1, *mseq2, penalty ); // nennnotame
-//						fprintf( stderr, "pscore (2) = %f\n\n", pscore );
-						break;
-					ErrorExit( "ERROR IN SOURCE FILE" );
-				}
-			}
+			fprintf( stderr, "  Calling PDP_ASH.pl %d-%d/%d    \r", i+1, j+1, njob );
+			pscore = callrash( i, j, mseq1, mseq2, equiv, strfiles[i], chainids[i], strfiles[j], chainids[j], alloclen );
+			off1 = off2 = 0;
+			break;
 			distancemtx[i][j] = pscore;
 #if SCOREOUT
 			fprintf( stderr, "score = %10.2f (%d,%d)\n", pscore, i, j );
@@ -901,7 +773,6 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 			{
 				alreadyoutput[j] = 1;
 				gappick0( seq[j], mseq2[0] );
-				fprintf( stdout, ">%d_%s-%s\n%s\n", j+1, strfiles[j], chainids[j], seq[j] );
 				strcpy( checkseq[j], seq[j] );
 			}
 			else
@@ -920,6 +791,10 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 #endif
 		}
 	}
+
+//	for( i=0; i<njob; i++ )
+//		fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, strfiles[i], chainids[i], checkseq[i] );
+
 	for( i=0; i<njob; i++ )
 	{
 		pscore = 0.0;
@@ -1052,6 +927,9 @@ int main( int argc, char *argv[] )
 	int alloclen;
 
 	arguments( argc, argv );
+#ifndef enablemultithread
+	nthread = 0;
+#endif
 
 	if( equivthreshold < 1 || 9 < equivthreshold )
 	{
