@@ -14,6 +14,30 @@ static int stdout_dist;
 static int store_localhom;
 static int store_dist;
 
+typedef struct _jobtable
+{
+	int i;
+	int j;
+} Jobtable;
+
+#ifdef enablemultithread
+typedef struct _thread_arg
+{
+	int thread_no;
+	int njob;
+	Jobtable *jobpospt;
+	char **name;
+	char **seq;
+	LocalHom **localhomtable;
+	double **distancemtx;
+	double *selfscore;
+	char ***bpp;
+	int alloclen;
+	pthread_mutex_t *mutex_counter;
+	pthread_mutex_t *mutex_stdout;
+} thread_arg_t;
+#endif
+
 static void t2u( char *seq )
 {
 	while( *seq )
@@ -225,81 +249,30 @@ static float recalllara( char **mseq1, char **mseq2, int alloclen )
 	return( value );
 }
 
-#if 0
-static float recallmxscarna( char **mseq1, char **mseq2, int m1, int m2, int alloclen )
-{
-	FILE *fp = NULL;
-	static char *ungap1 = NULL;
-	static char *ungap2 = NULL;
-	static char *ori1 = NULL;
-	static char *ori2 = NULL;
-//	int res;
-	static char com[10000];
-	float value;
 
-
-	if( ungap1 == NULL )
-	{
-		ungap1 = AllocateCharVec( alloclen );
-		ungap2 = AllocateCharVec( alloclen );
-		ori1 = AllocateCharVec( alloclen );
-		ori2 = AllocateCharVec( alloclen );
-	}
-
-	sprintf( com, "_%d-_%d.fasta", m1, m2 );
-	fp = fopen( com, "r" );
-	if( fp == NULL )
-	{
-		fprintf( stderr, "Cannot open %s.\n", com );
-		exit( 1 );
-	}
-
-	strcpy( ori1, *mseq1 );
-	strcpy( ori2, *mseq2 );
-
-	fgets( com, 999, fp );
-	load1SeqWithoutName_new( fp, *mseq1 );
-	fgets( com, 999, fp );
-	load1SeqWithoutName_new( fp, *mseq2 );
-
-	gappick0( ungap1, *mseq1 );
-	gappick0( ungap2, *mseq2 );
-	t2u( ungap1 );
-	t2u( ungap2 );
-	t2u( ori1 );
-	t2u( ori2 );
-
-	if( strcmp( ungap1, ori1 ) || strcmp( ungap2, ori2 ) )
-	{
-		fprintf( stderr, "SEQUENCE CHANGED!!\n" );
-		fprintf( stderr, "*mseq1  = %s\n", *mseq1 );
-		fprintf( stderr, "ungap1  = %s\n", ungap1 );
-		fprintf( stderr, "ori1    = %s\n", ori1 );
-		fprintf( stderr, "*mseq2  = %s\n", *mseq2 );
-		fprintf( stderr, "ungap2  = %s\n", ungap2 );
-		fprintf( stderr, "ori2    = %s\n", ori2 );
-		exit( 1 );
-	}
-
-	value = (float)naivepairscore11( *mseq1, *mseq2, penalty );
-
-	fclose( fp );
-
-	return( value );
-}
-#endif
-
-static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, char **bpp2, int alloclen )
+static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, char **bpp2, int alloclen, int i, int j )
 {
 	FILE *fp;
 	int res;
-	static char com[10000];
+	char *com;
 	float value;
+	char *dirname;
 
-	fp = fopen( "_bpporg", "w" );
+
+	dirname = calloc( 100, sizeof( char ) );
+	com = calloc( 1000, sizeof( char ) );
+	sprintf( dirname, "_%d-%d", i, j );
+	sprintf( com, "rm -rf %s", dirname );
+	system( com );
+	sprintf( com, "mkdir %s", dirname );
+	system( com );
+
+
+	sprintf( com, "%s/_bpporg", dirname );
+	fp = fopen( com, "w" );
 	if( !fp )
 	{
-		fprintf( stderr, "Cannot write to _bpporg\n" );
+		fprintf( stderr, "Cannot write to %s/_bpporg\n", dirname );
 		exit( 1 );
 	}
 	fprintf( fp, ">a\n" );
@@ -311,14 +284,17 @@ static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, c
 		fprintf( fp, "%s", *bpp2++ );
 	fclose( fp );
 
-	system( "tr -d '\\r' < _bpporg > _bpp" ); // for cygwin, wakaran
+	sprintf( com, "tr -d '\\r' < %s/_bpporg > %s/_bpp", dirname, dirname );
+	system( com ); // for cygwin, wakaran
 
 	t2u( *mseq1 );
 	t2u( *mseq2 );
-	fp = fopen( "_mxscarnainorg", "w" );
+
+	sprintf( com, "%s/_mxscarnainorg", dirname );
+	fp = fopen( com, "w" );
 	if( !fp )
 	{
-		fprintf( stderr, "Cannot open _mxscarnainorg\n" );
+		fprintf( stderr, "Cannot open %s/_mxscarnainorg\n", dirname );
 		exit( 1 );
 	}
 	fprintf( fp, ">1\n" );
@@ -329,9 +305,10 @@ static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, c
 	write1seq( fp, *mseq2 );
 	fclose( fp );
 
-	system( "tr -d '\\r' < _mxscarnainorg > _mxscarnain" ); // for cygwin, wakaran
+	sprintf( com, "tr -d '\\r' < %s/_mxscarnainorg > %s/_mxscarnain", dirname, dirname );
+	system( com ); // for cygwin, wakaran
 
-	sprintf( com, "env PATH=%s mxscarnamod -readbpp _mxscarnain > _mxscarnaout 2>_dum", whereispairalign );
+	sprintf( com, "cd %s; %s/mxscarnamod -readbpp _mxscarnain > _mxscarnaout 2>_dum", dirname, whereispairalign );
 	res = system( com );
 	if( res )
 	{
@@ -339,10 +316,12 @@ static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, c
 		exit( 1 );
 	}
 
-	fp = fopen( "_mxscarnaout", "r" );
+	sprintf( com, "%s/_mxscarnaout", dirname );
+
+	fp = fopen( com, "r" );
 	if( !fp )
 	{
-		fprintf( stderr, "Cannot open _mxscarnaout\n" );
+		fprintf( stderr, "Cannot open %s/_mxscarnaout\n", dirname );
 		exit( 1 );
 	}
 
@@ -358,6 +337,18 @@ static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, c
 
 	value = (float)naivepairscore11( *mseq1, *mseq2, penalty );
 
+	sprintf( com, "rm -rf %s > /dev/null 2>&1", dirname );
+	if( system( com ) )
+	{
+		fprintf( stderr, "retrying to rmdir\n" );
+		usleep( 2000 );
+		system( com );
+	}
+
+	free( dirname );
+	free( com );
+
+
 	return( value );
 }
 
@@ -476,6 +467,7 @@ void arguments( int argc, char *argv[] )
 {
     int c;
 
+	nthread = 1;
 	foldalignopt[0] = 0;
 	laraparams = NULL;
 	inputfile = NULL;
@@ -601,6 +593,11 @@ void arguments( int argc, char *argv[] )
 					fprintf( stderr, "laraparams = %s\n", laraparams );
 					--argc; 
 					goto nextoption;
+				case 'C':
+					nthread = atoi( *++argv );
+					fprintf( stderr, "nthread = %d\n", nthread );
+					--argc; 
+					goto nextoption;
 				case 'c':
 					stdout_dist = 1;
 					break;
@@ -684,9 +681,6 @@ void arguments( int argc, char *argv[] )
 				case 'V':
 					alg = 'V';
 					break;
-				case 'C':
-					alg = 'C';
-					break;
 				case 'F':
 					use_fft = 1;
 					break;
@@ -743,11 +737,6 @@ void arguments( int argc, char *argv[] )
 		fprintf( stderr, "conflicting options : o, m or u\n" );
 		exit( 1 );
 	}
-	if( alg == 'C' && outgap == 0 )
-	{
-		fprintf( stderr, "conflicting options : C, o\n" );
-		exit( 1 );
-	}
 }
 
 int countamino( char *s, int end )
@@ -758,7 +747,216 @@ int countamino( char *s, int end )
 	return( val );
 }
 
-static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, char **mseq1, char **mseq2, double *effarr, int alloclen )
+#if enablemultithread
+static void *athread( void *arg )
+{
+	thread_arg_t *targ = (thread_arg_t *)arg;
+	int i, j;
+	int clus1, clus2;
+	int off1, off2;
+	int intdum;
+	double bunbo;
+	float pscore = 0.0; // by D.Mathog
+	double *effarr;
+	double *effarr1;
+	double *effarr2;
+	char **pair;
+	char *indication1, *indication2;
+	char **mseq1, **mseq2;
+	char **aseq;
+
+// thread_arg
+	int thread_no = targ->thread_no;
+	int njob = targ->njob;
+	Jobtable *jobpospt = targ->jobpospt;
+	char **name = targ->name;
+	char **seq = targ->seq;
+	LocalHom **localhomtable = targ->localhomtable;
+	double **distancemtx = targ->distancemtx;
+	double *selfscore = targ->selfscore;
+	char ***bpp = targ->bpp;
+	int alloclen = targ->alloclen;
+
+//	fprintf( stderr, "thread %d start!\n", thread_no );
+
+	effarr = AllocateDoubleVec( njob );
+	for( i=0; i<njob; i++ ) effarr[i] = 1.0;
+	effarr1 = AllocateDoubleVec( njob );
+	effarr2 = AllocateDoubleVec( njob );
+	indication1 = AllocateCharVec( 150 );
+	indication2 = AllocateCharVec( 150 );
+	pair = AllocateCharMtx( njob, njob );
+	for( i=0; i<njob; i++ ) for( j=0; j<njob; j++ ) pair[i][j] = 0;
+	for( i=0; i<njob; i++ ) pair[i][i] = 1;
+	mseq1 = AllocateCharMtx( njob, 0 );
+	mseq2 = AllocateCharMtx( njob, 0 );
+	aseq = AllocateCharMtx( njob, alloclen+10 );
+
+	while( 1 )
+	{
+		pthread_mutex_lock( targ->mutex_counter );
+		j = jobpospt->j;
+		i = jobpospt->i;
+		j++;
+		if( j == njob )
+		{
+			i++;
+			j = i + 1;
+			if( i == njob-1 )
+			{
+//				fprintf( stderr, "thread %d end!\n", thread_no );
+				pthread_mutex_unlock( targ->mutex_counter );
+				return( NULL );
+			}
+		}
+		jobpospt->j = j;
+		jobpospt->i = i;
+		pthread_mutex_unlock( targ->mutex_counter );
+
+
+		if( j == i+1 || j % 100 == 0 ) 
+		{
+			fprintf( stderr, "% 5d / %d (by thread %3d) \r", i, njob, thread_no );
+//			fprintf( stderr, "% 5d - %5d / %d (thread %d)\n", i, j, njob, thread_no );
+		}
+
+
+		if( strlen( seq[i] ) == 0 || strlen( seq[j] ) == 0 )
+		{
+			if( store_dist ) distancemtx[i][j] = 2.0;
+			if( stdout_dist) 
+			{
+				pthread_mutex_lock( targ->mutex_stdout );
+				fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, 2.0 );
+				pthread_mutex_unlock( targ->mutex_stdout );
+			}
+			continue;
+		}
+
+		strcpy( aseq[i], seq[i] );
+		strcpy( aseq[j], seq[j] );
+		clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
+		clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+//		fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
+//		fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
+	
+#if 0
+		fprintf( stderr, "group1 = %.66s", indication1 );
+		fprintf( stderr, "\n" );
+		fprintf( stderr, "group2 = %.66s", indication2 );
+		fprintf( stderr, "\n" );
+#endif
+//		for( l=0; l<clus1; l++ ) fprintf( stderr, "## STEP-eff for mseq1-%d %f\n", l, effarr1[l] );
+
+		if( use_fft )
+		{
+			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+//			fprintf( stderr, "pscore (fft) = %f\n", pscore );
+			off1 = off2 = 0;
+		}
+		else
+		{
+			switch( alg )
+			{
+				case( 'L' ):
+					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+					L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+					break;
+				case( 'A' ):
+					pscore = G__align11( mseq1, mseq2, alloclen );
+					off1 = off2 = 0;
+					break;
+				case( 'N' ):
+					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+					genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+					break;
+				case( 't' ):
+					pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+					off1 = off2 = 0;
+					break;
+				case( 's' ):
+					pscore = callmxscarna_giving_bpp( mseq1, mseq2, bpp[i], bpp[j], alloclen, i, j );
+					off1 = off2 = 0;
+					break;
+#if 0 
+				case( 'a' ):
+					pscore = Aalign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen );
+					off1 = off2 = 0;
+					break;
+				case( 'K' ):
+					pscore = genG__align11( mseq1, mseq2, alloclen );
+					off1 = off2 = 0;
+					break;
+				case( 'H' ):
+					pscore = recallpairfoldalign( mseq1, mseq2, i, j, &off1, &off2, alloclen );
+					break;
+				case( 'B' ):
+				case( 'T' ):
+					pscore = recalllara( mseq1, mseq2, alloclen );
+					off1 = off2 = 0;
+					break;
+				case( 'M' ):
+					pscore = MSalign11( mseq1, mseq2, alloclen );
+					break;
+#endif
+				default:
+					ErrorExit( "\n\nERROR IN SOURCE FILE\n\n" );
+			}
+		}
+
+		if( alg == 't' || ( mseq1[0][0] != 0 && mseq2[0][0] != 0  ) ) // 't' no jouken ha iranai to omou. if( ( mseq1[0][0] != 0 && mseq2[0][0] != 0  ) )
+		{
+#if SCOREOUT
+			fprintf( stderr, "score = %10.2f (%d,%d)\n", pscore, i, j );
+#endif
+			if( !store_localhom )
+				;
+			else if( alg == 'H' )
+				putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+			else if( alg != 'S' && alg != 'V' )
+			{
+				putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+			}
+
+			if( (bunbo=MIN( selfscore[i], selfscore[j] )) == 0.0 || bunbo < pscore )
+				pscore = 2.0;
+			else
+				pscore = ( 1.0 - pscore / bunbo ) * 2.0;
+		}
+		else
+		{
+			pscore = 2.0;
+		}
+
+#if 1 // mutex
+		if( stdout_align )
+		{
+			pthread_mutex_lock( targ->mutex_stdout );
+			if( alg != 't' )
+			{
+				fprintf( stdout, "sequence %d - sequence %d, pairwise distance = %10.5f\n", i+1, j+1, pscore );
+				fprintf( stdout, ">%s\n", name[i] );
+				write1seq( stdout, mseq1[0] );
+				fprintf( stdout, ">%s\n", name[j] );
+				write1seq( stdout, mseq2[0] );
+				fprintf( stdout, "\n" );
+			}
+			pthread_mutex_unlock( targ->mutex_stdout );
+		}
+		if( stdout_dist )
+		{
+			pthread_mutex_lock( targ->mutex_stdout );
+			if( j == i+1 ) fprintf( stdout, "%d %d d=%.3f\n", i+1, i+1, 0.0 );
+			fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, pscore );
+			pthread_mutex_unlock( targ->mutex_stdout );
+		}
+#endif // mutex
+		if( store_dist) distancemtx[i][j] = pscore;
+	}
+}
+#endif
+
+static void pairalign( char **name, int nlen[M], char **seq, char **aseq, char **mseq1, char **mseq2, double *effarr, int alloclen )
 {
 	int i, j, ilim;
 	int clus1, clus2;
@@ -766,10 +964,10 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 	float pscore = 0.0; // by D.Mathog
 	static char *indication1, *indication2;
 	FILE *hat2p, *hat3p;
-	static double **distancemtx;
-	static double *selfscore;
-	static double *effarr1 = NULL;
-	static double *effarr2 = NULL;
+	double **distancemtx;
+	double *selfscore;
+	double *effarr1;
+	double *effarr2;
 	char *pt;
 	char *hat2file = "hat2";
 	LocalHom **localhomtable, *tmpptr;
@@ -797,19 +995,17 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 		}
 	}
 
-	if( effarr1 == NULL ) 
-	{
-		if( store_dist ) distancemtx = AllocateDoubleMtx( njob, njob );
-		selfscore = AllocateDoubleVec( njob );
-		effarr1 = AllocateDoubleVec( njob );
-		effarr2 = AllocateDoubleVec( njob );
-		indication1 = AllocateCharVec( 150 );
-		indication2 = AllocateCharVec( 150 );
+	if( store_dist ) distancemtx = AllocateDoubleMtx( njob, njob );
+	else distancemtx = NULL;
+	selfscore = AllocateDoubleVec( njob );
+	effarr1 = AllocateDoubleVec( njob );
+	effarr2 = AllocateDoubleVec( njob );
+	indication1 = AllocateCharVec( 150 );
+	indication2 = AllocateCharVec( 150 );
 #if 0
 #else
-		pair = AllocateCharMtx( njob, njob );
+	pair = AllocateCharMtx( njob, njob );
 #endif
-	}
 
 #if 0
 	fprintf( stderr, "##### fftwinsize = %d, fftthreshold = %d\n", fftWinSize, fftThreshold );
@@ -863,162 +1059,186 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 
 	}
 
-	ilim = njob - 1;
-	for( i=0; i<ilim; i++ ) 
+#if enablemultithread
+	if( nthread > 0 )
 	{
-		if( stdout_dist) fprintf( stdout, "%d-%d d=%.3f\n", i+1, i+1, 0.0 );
-		fprintf( stderr, "% 5d / %d\r", i, njob );
-		for( j=i+1; j<njob; j++ )
+		Jobtable jobpos;
+		pthread_t *handle;
+		pthread_mutex_t mutex_counter;
+		pthread_mutex_t mutex_stdout;
+		thread_arg_t *targ;
+
+		jobpos.i = 0;
+		jobpos.j = 0;
+
+		targ = calloc( nthread, sizeof( thread_arg_t ) );
+		handle = calloc( nthread, sizeof( pthread_t ) );
+		pthread_mutex_init( &mutex_counter, NULL );
+		pthread_mutex_init( &mutex_stdout, NULL );
+
+		for( i=0; i<nthread; i++ )
 		{
+			targ[i].thread_no = i;
+			targ[i].njob = njob;
+			targ[i].jobpospt = &jobpos;
+			targ[i].name = name;
+			targ[i].seq = seq;
+			targ[i].localhomtable = localhomtable;
+			targ[i].distancemtx = distancemtx;
+			targ[i].selfscore = selfscore;
+			targ[i].bpp = bpp; 
+			targ[i].alloclen = alloclen;
+			targ[i].mutex_counter = &mutex_counter;
+			targ[i].mutex_stdout = &mutex_stdout;
 
-			if( strlen( seq[i] ) == 0 || strlen( seq[j] ) == 0 )
-			{
-				if( store_dist ) distancemtx[i][j] = 2.0;
-				if( stdout_dist) fprintf( stdout, "%d-%d d=%.3f\n", i+1, j+1, 2.0 );
-				continue;
-			}
+//			athread( (void *)targ );
+			pthread_create( handle+i, NULL, athread, (void *)(targ+i) );
+//			pthread_create( handle+i, NULL, bthread, (void *)(targ+i) );
+		}
 
-			strcpy( aseq[i], seq[i] );
-			strcpy( aseq[j], seq[j] );
-			clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
-			clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
-	//		fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
-	//		fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
-	
-#if 0
-			fprintf( stderr, "group1 = %.66s", indication1 );
-			fprintf( stderr, "\n" );
-			fprintf( stderr, "group2 = %.66s", indication2 );
-			fprintf( stderr, "\n" );
+
+		for( i=0; i<nthread; i++ )
+		{
+			pthread_join( handle[i], NULL );
+		}
+		pthread_mutex_destroy( &mutex_counter );
+		pthread_mutex_destroy( &mutex_stdout );
+	}
+	else
 #endif
-	//		for( l=0; l<clus1; l++ ) fprintf( stderr, "## STEP-eff for mseq1-%d %f\n", l, effarr1[l] );
+	{
+		ilim = njob - 1;
+		for( i=0; i<ilim; i++ ) 
+		{
+			if( stdout_dist) fprintf( stdout, "%d %d d=%.3f\n", i+1, i+1, 0.0 );
+			fprintf( stderr, "% 5d / %d\r", i, njob );
+			fflush( stderr );
+			for( j=i+1; j<njob; j++ )
+			{
 	
-#if 1
-			if( use_fft )
-			{
-				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
-//				fprintf( stderr, "pscore (fft) = %f\n", pscore );
-				off1 = off2 = 0;
-			}
-			else
-#endif
-			{
-				switch( alg )
+				if( strlen( seq[i] ) == 0 || strlen( seq[j] ) == 0 )
 				{
-					case( 'a' ):
-						pscore = Aalign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen );
-						off1 = off2 = 0;
-						break;
-					case( 't' ):
-						pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
-						off1 = off2 = 0;
-						break;
-					case( 'A' ):
-						pscore = G__align11( mseq1, mseq2, alloclen );
-						off1 = off2 = 0;
-						break;
-#if 0
-					case( 'V' ):
-						pscore = VAalign11( mseq1, mseq2, alloclen, &off1, &off2, localhomtable[i]+j );
-						fprintf( stderr, "i,j = %d,%d, score = %f\n", i,j, pscore );
-						break;
-					case( 'S' ):
-						fprintf( stderr, "aligning %d-%d\n", i, j );
-						pscore = suboptalign11( mseq1, mseq2, alloclen, &off1, &off2, localhomtable[i]+j );
-						fprintf( stderr, "i,j = %d,%d, score = %f\n", i,j, pscore );
-						break;
-#endif
-					case( 'N' ):
-						pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
-						genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
-//						fprintf( stderr, "pscore = %f\n", pscore );
-						break;
-					case( 'K' ):
-						pscore = genG__align11( mseq1, mseq2, alloclen );
-						off1 = off2 = 0;
-						break;
-					case( 'L' ):
-						pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
-						L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
-//						fprintf( stderr, "pscore (1) = %f\n", pscore );
-//						pscore = (float)naivepairscore11( *mseq1, *mseq2, penalty ); // nennnotame
-//						fprintf( stderr, "pscore (2) = %f\n\n", pscore );
-						break;
-					case( 'H' ):
-						pscore = recallpairfoldalign( mseq1, mseq2, i, j, &off1, &off2, alloclen );
-						break;
-					case( 'B' ):
-					case( 'T' ):
-						pscore = recalllara( mseq1, mseq2, alloclen );
-						off1 = off2 = 0;
-//						fprintf( stderr, "lara, pscore = %f\n", pscore );
-						break;
-					case( 's' ):
-						pscore = callmxscarna_giving_bpp( mseq1, mseq2, bpp[i], bpp[j], alloclen );
-//						pscore = G__align11_noalign( amino_disLN, -1200, -60, mseq1, mseq2, alloclen );
-						off1 = off2 = 0;
-//						fprintf( stderr, "scarna, pscore = %f\n", pscore );
-						break;
-					case( 'M' ):
-//						pscore = MSalign11( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL );
-						pscore = MSalign11( mseq1, mseq2, alloclen );
-//						fprintf( stderr, "pscore (M)= %f\n", pscore );
-						break;
-						ErrorExit( "ERROR IN SOURCE FILE" );
+					if( store_dist ) distancemtx[i][j] = 2.0;
+					if( stdout_dist) fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, 2.0 );
+					continue;
 				}
-			}
-
-			if( alg == 't' || ( mseq1[0][0] != 0 && mseq2[0][0] != 0  ) ) // 't' no jouken ha iranai to omou. if( ( mseq1[0][0] != 0 && mseq2[0][0] != 0  ) )
-			{
-#if SCOREOUT
-				fprintf( stderr, "score = %10.2f (%d,%d)\n", pscore, i, j );
-#endif
-//				fprintf( stderr, "pslocal = %d\n", pslocal );
-//				offset = makelocal( *mseq1, *mseq2, pslocal );
+	
+				strcpy( aseq[i], seq[i] );
+				strcpy( aseq[j], seq[j] );
+				clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
+				clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+	//			fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
+	//			fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
+		
 #if 0
-				fprintf( stderr, "off1 = %d, off2 = %d\n", off1, off2 );
+				fprintf( stderr, "group1 = %.66s", indication1 );
+				fprintf( stderr, "\n" );
+				fprintf( stderr, "group2 = %.66s", indication2 );
+				fprintf( stderr, "\n" );
 #endif
-
-				if( !store_localhom )
-					;
-				else if( alg == 'H' )
-					putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
-				else if( alg != 'S' && alg != 'V' )
-					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
-
-
-				if( (bunbo=MIN( selfscore[i], selfscore[j] )) == 0.0 || bunbo < pscore )
-					pscore = 2.0;
+	//			for( l=0; l<clus1; l++ ) fprintf( stderr, "## STEP-eff for mseq1-%d %f\n", l, effarr1[l] );
+	
+				if( use_fft )
+				{
+					pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+//					fprintf( stderr, "pscore (fft) = %f\n", pscore );
+					off1 = off2 = 0;
+				}
 				else
-					pscore = ( 1.0 - pscore / bunbo ) * 2.0;
-			}
-			else
-			{
-				pscore = 2.0;
-			}
-
-			if( stdout_align )
-			{
-				if( alg != 't' )
 				{
-					fprintf( stdout, "sequence %d - sequence %d, pairwise score = %.0f\n", i+1, j+1, pscore );
-					fprintf( stdout, ">%s\n", name[i] );
-					write1seq( stdout, mseq1[0] );
-					fprintf( stdout, ">%s\n", name[j] );
-					write1seq( stdout, mseq2[0] );
-					fprintf( stdout, "\n" );
+					switch( alg )
+					{
+						case( 'a' ):
+							pscore = Aalign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen );
+							off1 = off2 = 0;
+							break;
+						case( 't' ):
+							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+							off1 = off2 = 0;
+							break;
+						case( 'A' ):
+							pscore = G__align11( mseq1, mseq2, alloclen );
+							off1 = off2 = 0;
+							break;
+						case( 'N' ):
+							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+							genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+							break;
+						case( 'K' ):
+							pscore = genG__align11( mseq1, mseq2, alloclen );
+							off1 = off2 = 0;
+							break;
+						case( 'L' ):
+							pscore = G__align11_noalign( amino_dis, penalty, penalty_ex, mseq1, mseq2, alloclen );
+							L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+							break;
+						case( 'H' ):
+							pscore = recallpairfoldalign( mseq1, mseq2, i, j, &off1, &off2, alloclen );
+							break;
+						case( 'B' ):
+						case( 'T' ):
+							pscore = recalllara( mseq1, mseq2, alloclen );
+							off1 = off2 = 0;
+							break;
+						case( 's' ):
+							pscore = callmxscarna_giving_bpp( mseq1, mseq2, bpp[i], bpp[j], alloclen, i, j );
+							off1 = off2 = 0;
+							break;
+						case( 'M' ):
+							pscore = MSalign11( mseq1, mseq2, alloclen );
+							break;
+						default:
+							ErrorExit( "ERROR IN SOURCE FILE" );
+					}
 				}
+	
+				if( alg == 't' || ( mseq1[0][0] != 0 && mseq2[0][0] != 0  ) ) // 't' no jouken ha iranai to omou. if( ( mseq1[0][0] != 0 && mseq2[0][0] != 0  ) )
+				{
+#if SCOREOUT
+					fprintf( stderr, "score = %10.2f (%d,%d)\n", pscore, i, j );
+#endif
+					if( !store_localhom )
+						;
+					else if( alg == 'H' )
+						putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+					else if( alg != 'S' && alg != 'V' )
+						putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+	
+	
+					if( (bunbo=MIN( selfscore[i], selfscore[j] )) == 0.0 || bunbo < pscore )
+						pscore = 2.0;
+					else
+						pscore = ( 1.0 - pscore / bunbo ) * 2.0;
+				}
+				else
+				{
+					pscore = 2.0;
+				}
+	
+				if( stdout_align )
+				{
+					if( alg != 't' )
+					{
+						fprintf( stdout, "sequence %d - sequence %d, pairwise distance = %10.5f\n", i+1, j+1, pscore );
+						fprintf( stdout, ">%s\n", name[i] );
+						write1seq( stdout, mseq1[0] );
+						fprintf( stdout, ">%s\n", name[j] );
+						write1seq( stdout, mseq2[0] );
+						fprintf( stdout, "\n" );
+					}
+				}
+				if( stdout_dist ) fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, pscore );
+				if( store_dist) distancemtx[i][j] = pscore;
 			}
-			if( stdout_dist ) fprintf( stdout, "%d-%d d=%.3f\n", i+1, j+1, pscore );
-			if( store_dist) distancemtx[i][j] = pscore;
 		}
 	}
 
+
 	if( store_dist )
 	{
 		hat2p = fopen( hat2file, "w" );
 		if( !hat2p ) ErrorExit( "Cannot open hat2." );
-		WriteHat2( hat2p, njob, name, distancemtx );
+		WriteHat2_pointer( hat2p, njob, name, distancemtx );
 		fclose( hat2p );
 	}
 
@@ -1026,7 +1246,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 	if( !hat3p ) ErrorExit( "Cannot open hat3." );
 	if( store_localhom )
 	{
-		fprintf( stderr, "##### writing hat3\n" );
+		fprintf( stderr, "\n\n##### writing hat3\n" );
 		ilim = njob-1;	
 		for( i=0; i<ilim; i++ ) 
 		{
@@ -1104,8 +1324,6 @@ static void WriteOptions( FILE *fp )
 		fprintf( fp, "Algorithm A+\n" );
 	else if( alg == 'S' ) 
 		fprintf( fp, "Apgorithm S\n" );
-	else if( alg == 'C' ) 
-		fprintf( fp, "Apgorithm A+/C\n" );
 	else
 		fprintf( fp, "Unknown algorithm\n" );
 
@@ -1132,18 +1350,21 @@ static void WriteOptions( FILE *fp )
 
 int main( int argc, char *argv[] )
 {
-	static int  nlen[M];	
-	static char name[M][B], **seq;
-	static char **mseq1, **mseq2;
-	static char **aseq;
-	static char **bseq;
-	static double *eff;
+	int  nlen[M];	
+	char **name, **seq;
+	char **mseq1, **mseq2;
+	char **aseq;
+	char **bseq;
+	double *eff;
 	int i;
 	FILE *infp;
 	char c;
 	int alloclen;
 
 	arguments( argc, argv );
+#ifndef enablemultithread
+	nthread = 0;
+#endif
 
 	if( inputfile )
 	{
@@ -1173,19 +1394,20 @@ int main( int argc, char *argv[] )
 		exit( 1 );
 	}
 
-	seq = AllocateCharMtx( njob, nlenmax*9+1 );
-	aseq = AllocateCharMtx( njob, nlenmax*9+1 );
-	bseq = AllocateCharMtx( njob, nlenmax*9+1 );
+	alloclen = nlenmax*2;
+	seq = AllocateCharMtx( njob, alloclen+10 );
+	aseq = AllocateCharMtx( njob, alloclen+10 );
+	bseq = AllocateCharMtx( njob, alloclen+10 );
 	mseq1 = AllocateCharMtx( njob, 0 );
 	mseq2 = AllocateCharMtx( njob, 0 );
-	alloclen = nlenmax*9;
+	name = AllocateCharMtx( njob, B );
 
 	eff = AllocateDoubleVec( njob );
 
 #if 0
 	Read( name, nlen, seq );
 #else
-	readData( infp, name, nlen, seq );
+	readData_pointer( infp, name, nlen, seq );
 #endif
 	fclose( infp );
 
@@ -1231,5 +1453,15 @@ int main( int argc, char *argv[] )
 	fprintf( stderr, "OSHIMAI\n" );
 #endif
 	SHOWVERSION;
+
+	if( stdout_dist && nthread > 1 )
+	{
+		fprintf( stderr, "\nThe order of distances is not identical to that in the input file, because of the parallel calculation.  Reorder them by yourself, using sort -n -k 2 | sort -n -k 1 -s\n" );
+	}
+	if( stdout_align && nthread > 1 )
+	{
+		fprintf( stderr, "\nThe order of pairwise alignments is not identical to that in the input file, because of the parallel calculation.  Reorder them by yourself.\n" );
+	}
+
 	return( 0 );
 }
