@@ -56,8 +56,8 @@ static void st_FinalGapCount( float *fgcp, int clus, char **seq, double *eff, in
 			
 		
 
-static int impalloclen = 0;
-static float **impmtx = NULL;
+static TLS int impalloclen = 0;
+static TLS float **impmtx = NULL;
 float part_imp_match_out_sc( int i1, int j1 )
 {
 //	fprintf( stderr, "impalloclen = %d\n", impalloclen );
@@ -271,10 +271,10 @@ void part_imp_rna( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1,
 void part_imp_match_init( float *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, LocalHom ***localhom )
 {
 	int dif, i, j, k1, k2, tmpint, start1, start2, end1, end2;
-	static int impalloclen = 0;
+	static TLS int impalloclen = 0;
 	char *pt;
-	static char *nocount1 = NULL;
-	static char *nocount2 = NULL;
+	static TLS char *nocount1 = NULL;
+	static TLS char *nocount2 = NULL;
 
 	if( impalloclen < lgth1 || impalloclen < lgth2 )
 	{
@@ -518,8 +518,10 @@ static void Atracking_localhom( float *impwmpt, float *lasthorizontalw, float *l
 						int *gapmap1, int *gapmap2 )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k;
-	char gap[] = "-";
+//	char gap[] = "-";
+	char *gap;
 	float wm;
+	gap = newgapstr;
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
@@ -629,7 +631,9 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
                         int **ijp, int icyc, int jcyc )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, lastk;
-	char gap[] = "-";
+//	char gap[] = "-";
+	char *gap;
+	gap = newgapstr;
 	float wm = 0.0;
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
@@ -747,25 +751,25 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 	float *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	static float mi, *m;
-	static int **ijp;
-	static int mpi, *mp;
-	static float *w1, *w2;
-	static float *match;
-	static float *initverticalw;    /* kufuu sureba iranai */
-	static float *lastverticalw;    /* kufuu sureba iranai */
-	static char **mseq1;
-	static char **mseq2;
-	static char **mseq;
-	static float *ogcp1;
-	static float *ogcp2;
-	static float *fgcp1;
-	static float *fgcp2;
-	static float **cpmx1;
-	static float **cpmx2;
-	static int **intwork;
-	static float **floatwork;
-	static int orlgth1 = 0, orlgth2 = 0;
+	static TLS float mi, *m;
+	static TLS int **ijp;
+	static TLS int mpi, *mp;
+	static TLS float *w1, *w2;
+	static TLS float *match;
+	static TLS float *initverticalw;    /* kufuu sureba iranai */
+	static TLS float *lastverticalw;    /* kufuu sureba iranai */
+	static TLS char **mseq1;
+	static TLS char **mseq2;
+	static TLS char **mseq;
+	static TLS float *ogcp1;
+	static TLS float *ogcp2;
+	static TLS float *fgcp1;
+	static TLS float *fgcp2;
+	static TLS float **cpmx1;
+	static TLS float **cpmx2;
+	static TLS int **intwork;
+	static TLS float **floatwork;
+	static TLS int orlgth1 = 0, orlgth2 = 0;
 	float fpenalty = (float)penalty;
 #if USE_PENALTY_EX
 	float fpenalty_ex = (float)penalty_ex;
